#include "search_application.h"
#include "mainwindow.h"
#include <QCommandLineParser>
#include <QDebug>

SearchApplication::SearchApplication(const QString &appId, int &argc, char **argv) : QtSingleApplication(appId,argc,argv)
{
    if(!isRunning()&&this->arguments().size()==2&&this->arguments().at(1)=="-q"){
        setFlag(1);
        return;
    }

    if(!isRunning()){
        initMainWindow();
        connect(this,&QtSingleApplication::messageReceived,[=](const QString &msg){
            this->parseCommand(msg, true);
        });
    }
    parseCommand(this->arguments().join(' '),!isRunning());
}

SearchApplication::~SearchApplication()
{
    if(m_mainWindow){
        delete m_mainWindow;
        m_mainWindow = nullptr;
    }
}

void SearchApplication::initMainWindow()
{
    if(!m_mainWindow){
        m_mainWindow = new MainWindow();
    }
}

void SearchApplication::parseCommand(const QString &msg, bool isPrimary)
{
    QCommandLineParser parser;
    QCommandLineOption quitOption = QCommandLineOption(QStringList()<<"q"<<"quit",tr("Quit"));


    parser.addOption(quitOption);
    parser.addHelpOption();

    if(isPrimary){
        const QStringList args = msg.split(' ');
        parser.process(args);

        if(parser.isSet(quitOption)){
            qDebug() << "quit main window";
            SougouInterface *sougouInterface = new SougouInterface();
            sougouInterface->showKeyboard(false);
            qApp->quit();
        }
    }else{
        parser.process(arguments());
        sendMessage(msg);
    }
}

